import * as globalMethods from 'quest-global';
import { element, by, browser, ExpectedConditions } from 'protractor';
import * as fs from 'fs';
import { Common } from '../support/common';
import { data } from 'jquery';
import { v4 as uuidv4 } from 'uuid';
import * as jsondata from '../../jsonfiles/his.json';
import * as labreportjson from '../../jsonfiles/labreport.json';
import * as hrspayloadjson from '../../jsonfiles/hrspayload.json';
import * as XLSX from 'xlsx';

var axios = require('axios');
var chai = require('chai');
var assert = chai.assert;
var crypto = require('crypto');

let requestHeaders = {
    'Content-Type': 'application/json',
};
let securityHeaders = {
    'X-CSRF-TOKEN': '123',
    'Content-Type': 'application/json',
    'Cookie': 'CSRF-TOKEN=123',
    'Access-Control-Allow-Origin': '*',
};

export class ApiAutomated {
    /**
     * @param dataFile
     * @param apiEndpoint
     */
    
    //generic global method for POST 
    async checkPOSTStatus(apiUrl, jsonfile) {
        try {
            let apiResponse = await axios.post(apiUrl, jsonfile, requestHeaders);
            let response = {
                "status": await apiResponse.status,
                "data": await apiResponse.data
            };
            return await response;
        }
        catch (error) {
            await globalMethods.log("***Error:" + await error + ", data:" + JSON.stringify(error));
        }
    }
    //generic global method for GET
    async checkGETStatus(apiUrl) {
        try {
            let apiResponse = await axios.get(apiUrl, requestHeaders);
            let response = {
                "status": await apiResponse.status,
                "data": await apiResponse.data
            };
            return await response;
        }
        catch (error) {
            await globalMethods.log("***Error:" + await error + ", data:" + JSON.stringify(error));
        }
    }
    //generic global method for DELETE
    async checkDeleteAPI(apiUrl) {
        try {
            let apiResponse = await axios.delete(apiUrl);
            let statusCode = await apiResponse.status;
            let data = await apiResponse.data;
            let response = {
                "status": statusCode,
                "data": data
            };
            return await data;
        }
        catch (error) {
            await globalMethods.log("***Error:" + await error + ", data:" + JSON.stringify(error));
        }
    } 
    //generic global method for PUT 
     async checkPUTStatus(apiUrl, jsonfile) {
        try {
            let apiResponse = await axios.put(apiUrl, jsonfile, requestHeaders);
            let response = {
                "status": await apiResponse.status,
                "data": await apiResponse.data
            };
            return await response;
        }
        catch (error) {
            await globalMethods.log("***Error:" + await error + ", data:" + JSON.stringify(error));
        }
    }
    //generic global method to validate error response
    async checkErrorPOSTAPI(apiUrl, jsonfile) {
        let status = await axios.post(apiUrl, jsonfile, requestHeaders).catch(function (error) {
            return error.response.status;
        });
        return status;
    }
    
    //generic global method to verify response code or data
    async verifyresponse(data, expected) {
        await globalMethods.verifyFailOnError(
            data,
            expected,
            "test failed: " + data,
            true           
        );
    }
    // guid generator
    async newGuid() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }

    //string generator 
    async makeRandomString(length) {
        var result = '';
        var characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        var charactersLength = characters.length;
        for (var i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() *
                charactersLength));
        }
        return result;
    }
  
    //Below lines of code to verify the payload by writing to a file
    async writetoFile(url) {
       var fs = require('fs');
        await fs.writeFile('jsonfiles/hrspayloadjson.txt', JSON.stringify(hrspayloadjson), function (err) {
            if (err) return console.log(err);
            console.log('could not create file');
        });       
    } 

     //convert excel to json
     async getExceltoJson(path){
        var XLSX = require('xlsx');
        var workbook = XLSX.readFile(path);
        var sheet_name_list = workbook.SheetNames;
        var xlData = await XLSX.utils.sheet_to_json(workbook.Sheets[sheet_name_list[0]]);
      return xlData;
    }

    //function to write json to excel
    async writeToExcel(path, jsonObj) {
        var json2xls = require('json2xls');
        const fs = require('fs');
        var xls = json2xls(jsonObj);
        fs.writeFileSync(path, xls, 'binary', (err) => {
            if (err) {
                  console.log("writeFileSync :", err);
             }
           console.log(path+" file is saved!");
        });
    }
     //function to compare two jsons
     async diff(json1, json2) {
        const result = {};
        if (Object.is(json1, json2)) {
            return undefined;
        }
        if (!json1 || typeof json2!== 'object') {
            return json2;
        }
        Object.keys(json1 || {}).concat(Object.keys(json2 || {})).forEach(key => {
            if(json2[key] !== json1[key] && !Object.is(json1[key], json2[key])) {
                result[key] = json2[key];               
            }
            if(typeof json2[key] === 'object' && typeof json1[key] === 'object') {
                const value = this.diff(json1[key], json2[key]);
                if (value !== undefined) {
                    result[key] = value;
                }
            }
        });
        return result;
    }

    //compare two excel sheets and print the difference
    async hasDiffsCompareTwoExcel(result) {    
        var diffsExist = false;
        for (let [key, value] of Object.entries(result)) {
            var a = await value;
            if(Object.keys(a).length > 0) {
                console.log("*****In row" + key + " there is a diff" + JSON.stringify(a));
                diffsExist = true; 
            } 
        }
        console.log("diffs exist: "+diffsExist);    
        return diffsExist;
    }   
}