import { browser, ExpectedConditions, ElementFinder, element, by } from 'protractor';
import { doesNotThrow, fail, strictEqual } from 'assert';
import 'jest-extended';
export class questglobal {

    getMobileElement(identifier: string, timeout?: number) {
        if (timeout === undefined) {
            timeout = 5000;
        }
        var mobile = <any>$(identifier);
        mobile.waitForDisplayed({ timeout: timeout });
        return mobile;
    }

    getElementByButtonText(buttonText: string) {
        return element(by.buttonText(buttonText));
    }

    getElementByClassName(className: string) {
        return element(by.className(className));
    }

    getElementByCss(css: string) {
        return element(by.css(css));
    }

    getElementByCssContainingText(css: string, text: string) {
        return element(by.cssContainingText(css, text));
    }

    getElementById(id: string) {
        return element(by.id(id));
    }

    getElementByName(name: string) {
        return element(by.name(name));
    }

    getElementByXpath(Xpath: string) {
        return element(by.xpath(Xpath));
    }

    /**
     * 
     * @param tagName HTML Object tag (i.e. button, input, a, div, etc)
     * @param attributeName Custom attribute name (i.e. data-ta)
     * @param attributeValue Custom attribute value
     */
    getElementByAttribute(tagName: String, attributeName: String, attributeValue: String) {
        var xpath = "//" + tagName + "[@" + attributeName + "='" + attributeValue + "']";
        return element(by.xpath(xpath));
    }

    /**
     * Mobile locators
     * @param numDaysOffset 
     * @param dateFormat 
     */

    /**
     * 
     * @param numDaysOffset positive number will return # days in future, negative number will return # days in past
     * @param dateFormat 
     */
    getDate(numDaysOffset: number, dateFormat: String) {
        var date = new Date;
        this.log("Original Date:" + date);
        this.log("Number of days offset:" + numDaysOffset);
        date.setDate(date.getDate() + numDaysOffset);
        this.log("Adjusted Date:" + date);
        var dateToReturn;
        if (dateFormat == "mm/dd/yyyy") {
            dateToReturn = (date.getMonth() + 1) + "/" + date.getDate() + "/" + date.getFullYear();
        }

        this.log("Date being returned:" + dateToReturn);
        return dateToReturn;
    }

    /**
       * Facilitates client-side navigation. Sets waitForAngularEnabled to false
       * and navigates to the url. Then waits for up to 8000 for validator to display
       *
       * @param {string} url - url to navigate to
       * @param {ElementFinder} validator - element to wait for after navigation
       * @api public
       */
    async goToURL(url: string, validator: ElementFinder) {
        browser.waitForAngularEnabled(false);
        await browser.get(url);
        await browser.wait(ExpectedConditions.presenceOf(validator), 8000, 'Element taking too long to appear in the DOM');
    }

    /**
     * Performs a Rest GET and return response text object at index 0. Used to connect
     * to internal datamanagement endpoint
     * 
     * @param {string} domain - url domain
     * @param {string} path - path to append to domain
     * @param {string} parameter - parameter to append to path and domain
     * @api public
     */
    getData(domain: string, path: string, parameter: string): string {
        var XMLHttpRequest = require("xmlhttprequest").XMLHttpRequest;
        var xmlHttp = new XMLHttpRequest();
        var endpoint = domain + "/" + path + "/" + parameter;
        console.log("Data url:" + endpoint);
        xmlHttp.open("GET", endpoint, false);
        xmlHttp.send(null);
        var json = JSON.parse(xmlHttp.responseText)[0];
        if (typeof json === 'undefined') {
            json = JSON.parse(xmlHttp.responseText);
        }
        return json;
    }

    /**
     * Validates element's presence/absence on page
     * 
     * @deprecated
     * @param {ElementFinder} element - element to verify presence/absence on page
     * @param {number} timeout - max amount of time to wait for element's presence/absence
     * @param {boolean} presence - determines whether verification is of element's presence or absence
     * @api public
     */
    async validateElement(element: ElementFinder, timeout: number, presence: boolean) {
        await this.verifyElement(element, timeout, presence);
    }

    /**
     * Validates text equal/not equal to comparison text
     * 
     * @deprecated
     * @param {string} text1 - first piece of text to compare
     * @param {string} text2 - second piece of text to compare
     * @param {boolean} presence - determines whether verification is of text equal/not equal
     * @api public
     */
    async validateText(text1: string, text2: string, presence: boolean) {
        await this.verifyText(text1, text2, presence);
    }

    /**
    * Validates text equal/not equal to comparison text
    * 
    * @deprecated
    * @param {string} text1 - first piece of text to compare
    * @param {string} text2 - second piece of text to compare
    * @param {boolean} presence - determines whether verification is of text equal/not equal
    * @api public
    */
    async validateTextContains(text1: string, text2: string, presence: boolean) {
        await this.verifyTextContains(text1, text2, presence);
    }

    /**
     * Sets waitForAngularEnabled to passed in parameter
     * 
     * @param {boolean} isAngularApp - sets protractor angularEnabled to true/false
     * @api public
     */
    isAngular(isAngularApp: boolean) {
        browser.waitForAngularEnabled(isAngularApp);
    }

    /**
       * 
       *  Waits for element presence/absence for up to timeout (parameter)
       * 
       *  Sample syntax   var retVal = await waitForElement (element, presence, timeout_In_Milliseconds);
       * 
       * 
       * @param element 
       * @param presence 
       * @param timeout 
       * 
       * Return Value: true/false
       */
    async waitForElement(element: ElementFinder, presence: boolean, timeout: number): Promise<boolean> {

        var methodName = "waitForElement";

        if (presence) {
            try {
                await browser.wait(ExpectedConditions.presenceOf(element), timeout);
                await browser.wait(ExpectedConditions.visibilityOf(element), timeout);
                return true;
            } catch (e) {
                console.log(methodName + e);
                return false;
            }
        }

        if (!presence) {
            try {
                await browser.wait(ExpectedConditions.stalenessOf(element), timeout);
                return true;
            } catch (e) {
                console.log(methodName + e);
                return false;
            }
        }

        return false;
    }

    async waitForElementScrollIntoView(element: ElementFinder, presence: boolean, timeout: number): Promise<boolean> {
        let elementExists = await this.waitForElement(await element, await presence, await timeout);
        if (await presence) {
            try {
                await this.executeScriptScrollIntoView(element);
            }
            catch (e) {
                console.log("Error occurred scrolling into view. Not a dealbreaker. Error:" + await e);
            }
        }
        return await elementExists;
    }


    /**
     * Selects option from dropdown
     * @param {ElementFinder} element - element select value from
     * @param {string} item - value to select
     * @param {number} timeout - maximum amount of time to wait before selecting value
     * @api public
     */
    async selectFromDropDown(ele: ElementFinder, item: string, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(ele), timeout, 'Element not found.');
        await this.waitForVisible(ele, timeout);
        await ele.click();
        await browser.wait(ExpectedConditions.presenceOf(element(by.cssContainingText('.mat-option', item))), timeout);
        await element(by.cssContainingText('.mat-option', item)).click();
    }

    /**
     * Selects option from dropdown
     * @param {ElementFinder} element - element select value from
     * @param {string} item - value to select
     * @param {number} timeout - maximum amount of time to wait before selecting value
     * @api public
     */
    async enterAndSelectFromAutoSuggest(ele: ElementFinder, item: string, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(ele), timeout, 'Element not found.');
        await this.waitForVisible(ele, timeout);
        await ele.sendKeys(item);
        await element(by.cssContainingText('mat-option', item)).click();
    }

    /**
     * Sends text to an element
     * @deprecated
     * @param {ElementFinder} element - element to enter text to
     * @param {string} text - text to send to element
     * @param {number} timeout - max amount of time to wait on element to be visible and ready for interaction, before sending text
     * @api public
     */
    async sendText(element: ElementFinder, keys: string, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element to enter keys: ' + keys + 'not found.');
        await element.isEnabled();
        await this.waitForVisible(element, timeout);
        await element.click();
        await element.clear();
        await element.sendKeys(keys);
    }

    sendKeys(element: ElementFinder, text: string, timeout: number, clearPreviousText: boolean){
        if (this.isWDIO()) { 
            return this.zzSendWDIOKeys(element, text, timeout, clearPreviousText);
        } 
        else {
            return this.zzSendProtractorKeys(element, text, timeout, clearPreviousText);
        }
    }

    /**
     * Sends keyboard keys
     * Supported keys reference documentation: https://w3c.github.io/webdriver/#keyboard-actions
     * @param element 
     * @param text 
     * @param timeout 
     * @param clearPreviousText 
     */
    zzSendWDIOKeys(element: any, text: string, timeout: number, clearPreviousText: boolean){
        console.log("Sending keys with WDIO")
        if(clearPreviousText){
            console.log("Clearing previous text and sending keys.");
            return element.setValue(text);
        }
        else{
            console.log("(NOT clearing previous text) sending keys.")
            return element.addValue(text);
        }
    }

    /**
     * Sends keys to an element, without first clearing text
     * @param element 
     * @param elementName 
     * @param timeout 
     */
    async zzSendProtractorKeys(element: ElementFinder, text: string, timeout: number, clearPreviousText: boolean) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element to enter text: ' + text + 'not found.');
        await element.isEnabled();
        await this.waitForVisible(element, timeout);
        await element.click();
        if (clearPreviousText) {
            await element.clear();
        }
        await element.sendKeys(text);
    }

    /**
     * Clicks an element
     * @param {ElementFinder} element - element to click
     * @param {String} elementName - element name for this.logger statement
     * @param {number} timeout - max amount of time to wait before clicking element
     */
    async click(element: ElementFinder, elementName: String, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element: ' + elementName + ' not found.');
        await element.isEnabled();
        await this.waitForVisible(element, 10000);
        await element.click();
        await this.log('Clicked element: ' + elementName + '.');
    }

    /**
     * Moves mouse over element
     * @param element 
     * @param elementName 
     * @param timeout 
     */
    async mouseMove(element: ElementFinder, elementName: String, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element: ' + elementName + ' not found.');
        await this.waitForVisible(element, timeout);
        await element.isEnabled();
        await element.isDisplayed();
        await browser.actions().mouseMove(element).perform();
    }

    /**
     * Moves mouse over element before clicking, subsequently clicks
     * @param element 
     * @param elementName 
     * @param timeout 
     */
    async mouseMoveClick(element: ElementFinder, elementName: String, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element: ' + elementName + ' not found.');
        await this.waitForVisible(element, timeout);
        await element.isEnabled();
        await element.isDisplayed();
        await browser
            .actions()
            .mouseMove(element)
            .click()
            .perform();
    }

    /**
     * Clicks element via JavaScript
     * @param element 
     * @param elementName 
     * @param timeout 
     */
    async executeScriptClick(element: ElementFinder, elementName: String, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element: ' + elementName + ' not found.');
        await this.waitForVisible(element, timeout);
        await browser.executeScript('arguments[0].click();', await element);
        console.log('Clicked element: ' + elementName + ' via JavaScript.');
    }

    /**
     * Brings element into view
     * @param element 
     * @param elementName 
     * @param timeout 
     */
    async executeScriptScrollIntoView(element: ElementFinder) {
        await console.log('BEGIN Scrolling to an element');
        await browser.executeScript('arguments[0].scrollIntoView({block: \'center\'})', await element);
        await console.log('END Scrolling to an element. The element should be in the center of the page vertically');
    }

    /**
     * Clears text from a textfield
     * @param {ElementFinder} element - element to clear text from
     * @param {string} this.logString - element name to this.log to console
     * @param {number} timeout - max amount of time to wait for element to appear on page
     * @api public
     */
    async clearText(element: ElementFinder, elementName: string, timeout: number) {
        await browser.wait(ExpectedConditions.presenceOf(element), timeout, 'Element: ' + elementName + 'not found.');
        await this.waitForVisible(element, timeout);
        await element.isEnabled();
        await element.clear();
    }

    /**
     * Switches to tab with inputted index
     * @param {number} tabIndex - index of tab to switch to
     */
    async switchTabs(tabIndex: number) {
        let handles = await browser.getAllWindowHandles();
        await browser.switchTo().window(handles[tabIndex]);
    }

    /**
     * Switches focus to frame, based on xpath input
     * @param xpath 
     */
    async switchToFrame(xpath: string) {
        var el = await browser.driver.findElement(by.xpath(xpath));
        await browser.switchTo().frame(el);
    }

    /**
     * Closes current tab
     */
    async closeTab() {
        await browser.driver.close();
    }

    /**
     * Returns boolean value of whether text equal/not equal to comparison text
     * 
     * @param {string} text1 - first piece of text to compare
     * @param {string} text2 - second piece of text to compare
     * @param {boolean} presence - determines whether verification is of text equal/not equal
     * @api public
     */
    async textContains(text1: string, text2: string) {
        return await text1.includes(text2);
    }

    /**
     * Gets the viewport size from the processed config file
     */
    getConfigBrowserSize() {
        return browser.params.viewport;
    }


    verifyFailOnError(actual: any, expected: any, failStr: string, match: boolean) {
        if (this.isWDIO()) { 
            return this.zzVerifyWDIOFailOnError(actual,expected);
        } else {
            return this.zzVerifyProtractorFailOnError(actual,expected,failStr,match);
        }

    }

    /**
     * 
     * @param actual 
     * @param expected 
     * @param failStr 
     */
    async zzVerifyProtractorFailOnError(actual: any, expected: any, failStr: string, match: boolean) {
        await console.log("Hi, message 1/4")
        var date = new Date();
        await console.log("[" + date.toLocaleTimeString() + "] " + "verifyFailOnError: Expected=" + expected + "  Actual=" + actual + "  String=" + failStr);

        //If match parameter is not passed in, default it to true
        if (typeof match === "undefined") {
            match = true;
        }

        //If match is equal to true, verify actual and expected do match, and fail on error
        if (match === true) {
            if (expected !== actual) {
                await console.log("Hi, message 2/4");
                // Not equal value or NOT equal type
                    await fail(actual, expected, failStr);
                }
                await console.log("Hi, message 3/4");
            return;
        }

        //If match is equal to false, verify actual and expected do not match, and fail on error
        if (expected === actual) {
            await fail(actual, expected, failStr);
        }

    }
    
    verifyElement(element: any, timeout: number, presence: boolean) {
        if (this.isWDIO()) { 
            return this.zzVerifyWDIOElement(element, timeout, presence);
        } 
        else {
            return this.zzVerifyProtractorElement(element, timeout, presence);
        }
    }
    /**
   * Verifies element's presence/absence on page
   * 
   * @param {ElementFinder} element - element to verify presence/absence on page
   * @param {number} timeout - max amount of time to wait for element's presence/absence
   * @param {boolean} presence - determines whether verification is of element's presence or absence
   * @api public
   */
    async zzVerifyProtractorElement(element: any, timeout: number, presence: boolean) {
        var isPresent = null;
        if (element instanceof ElementFinder) {
            var chai = require('chai');
            chai.use(require('chai-as-promised'));
            var expect = chai.expect;
            try {
                console.log("We're getting in the try loop");
                if (presence) {
                    console.log("Expecting element to be present. Waiting up to five seconds for element to appear. Time Started: " + new Date());
                    await browser.wait(ExpectedConditions.presenceOf(element), timeout);
                    console.log("Finished waiting for element. Time Finished: " + new Date());
                }
                isPresent = await element.isPresent();
            } catch (err) {
                throw Error(err + 'Verification failed');
            }
            console.log("IsPresent = " + isPresent);
            console.log("Actual element presence = " + isPresent);
            console.log("Expected element presence = " + presence);
            return await expect(isPresent).to.equal(presence);
        }
    }

    async zzVerifyWDIOElement(element: any, timeout: number, presence: boolean) {
        require('expect-webdriverio')
        try {
            if (presence) {
                console.log("Expecting WDIO element to be present. Waiting timeout to appear. Time Started: " + new Date());
                element.waitForExist({ timeout: timeout });
                console.log("Finished waiting for element. Time Finished: " + new Date());
            }
        }
        catch (err) {
            throw Error(err + 'Verification failed')
        }
        console.log("Expected element presence = " + presence);
        var isPresent = await element.isDisplayed();
        return await expect(isPresent).toEqual(presence);
    }

    zzVerifyWDIOFailOnError(actual: any, expected: any, match?: boolean) {

        console.log("calling verifyWDIOFailOnError(actual="+actual+" expected="+expected);

        require('expect-webdriverio');
        expect(actual).toEqual(expected);
    }


    /**
     * Verifies text equal/not equal to comparison text
     * 
     * @param {string} text1 - first piece of text to compare
     * @param {string} text2 - second piece of text to compare
     * @param {boolean} presence - determines whether verification is of text equal/not equal
     * @api public
     */
    async verifyText(text1: string, text2: string, presence: boolean) {
        var chai = require('chai');
        chai.use(require('chai-as-promised'));
        var expect = chai.expect;
        if (presence) {
            try {
                return await expect(text1).to.equal(text2);
            } catch (err) {
                throw Error("Verification failed: '" + text1 + "' does not match '" + text2 + "'.");
            }
        }
        return await expect(text1).to.not.equal(text2);
    }

    /**
    * Verifies text equal/not equal to comparison text
    * 
    * @param {string} text1 - first piece of text to compare
    * @param {string} text2 - second piece of text to compare
    * @param {boolean} presence - determines whether verification is of text equal/not equal
    * @api public
    */
    async verifyTextContains(text1: string, text2: string, presence: boolean) {
        var chai = require('chai');
        chai.use(require('chai-as-promised'));
        var expect = chai.expect;
        if (presence) {
            try {
                return await expect(text1).to.include(text2);
            } catch (err) {
                throw Error("Verification failed: '" + await text1 + "' does not include '" + await text2 + "'.");
            }
        }
        return await expect(text1).to.not.include(text2);
    }

    /**
     * Prepends this.log statement with timestamp
     * @param this.logMsg 
     */
    async log(logMsg: string) {

        var date = new Date();
        console.log("[" + date.toLocaleTimeString() + "] " + logMsg);

    }

    /**
     * Waits for element to be visible, up to specified number of seconds
     * @param ele 
     * @param timeout 
     */
    async waitForVisible(ele: ElementFinder, timeout: number) {
        await browser.wait(await ExpectedConditions.visibilityOf(ele), timeout);
    }

    /**
     * Waits for element to be clickable, up to specified number of seconds
     * @param ele
     * @param timeout 
     */
    async waitForClickable(ele: ElementFinder, timeout: number) {
        await browser.wait(await ExpectedConditions.elementToBeClickable(ele), timeout);
    }

    /**
     * Taps a mobile element
     */
    async tap(ele) {
        ele.touchAction('tap');
    }

    isWDIO() {
        console.log("Args:" + process.argv.toString());
        if (process.argv.toString().includes("@wdio")) {
            console.log("Mobile=true");
            return true;
        } else {
            console.log("Mobile=false");
            return false;
        }

    }


}