import { by, element, browser} from 'protractor';
import * as globalMethods from 'quest-global'
let timeout=9500;
var browserSize;

export class Login {
  // Getter Functions:
  getUserName() {
    return element(by.id('username'));
  }

  getPassword() {
    return element(by.id('password'));
  }

  getLogin() {
    return element(by.buttonText('Login'));
  }

  getHelp() {
    return element(by.id('help'));
  }

  getForgotUsername() {
    //return globalMethods.getElementById('forgotUserName');
    return globalMethods.getElementByXpath("//input[@aria-label='forgotUserName']");
    
  }

  getFAQ() {
    return element(by.id('FAQ'));
  }
 
  getContactUsLink(){
    return globalMethods.getElementByXpath("//a[contains(text(),'Contact Us')]");
  }
  getMessageQuestMobile() {
    //return element(by.id('contactDialog'));
    return globalMethods.getElementByXpath("//button[@aria-label='Message Quest']");
  }
 

  //  Reusable functions:

  /**
   * procedureEnterSignIn method takes two parameters userName and password. It enters username and password into respective fields on sign in page.
   * @param userName 
   * @param password 
   */
  async procedureEnterSignIn(userName, password){
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getUserName(),true,timeout);
    globalMethods.waitForVisible(await this.getUserName(),timeout);
    await browser.driver.findElement(by.id("username")).sendKeys(userName);
    //await this.getUserName().sendKeys(userName);
    await this.getPassword().sendKeys(password);
    await browser.waitForAngularEnabled(true);
  }
  /**
   * clickLogin method clicks on login button available on myquest login page.
   * This method supports for desktop and mobile viewport.
   */
  async clickLogin(){
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getLogin(), true, timeout);
    await globalMethods.click(await this.getLogin(), 'Clicked on login', timeout);
    await globalMethods.waitForElement(await this.getLogin(),false,timeout);
    await globalMethods.log("Made it to the end of logging in");
    await browser.waitForAngularEnabled(true);
  }

  async clickHelp(){
    await globalMethods.click(await this.getHelp(), 'Help', timeout);
  }
  async clickFAQ(){
    await globalMethods.click(await this.getFAQ(), 'FAQ', timeout);
  }
/**
 * clickForgotUsername method clicks on 'Forgot Username' link on sign in page.
 */
  async clickForgotUsername() {
    await browser.refresh();
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getForgotUsername(), true, timeout);
    await globalMethods.waitForVisible(await this.getForgotUsername(), timeout);
    await globalMethods.click(await this.getForgotUsername(), 'Forgot User Name', timeout);  
    await browser.waitForAngularEnabled(true);
  }
  /**
   * clickContactUsLink method clicks on Contact Us link on CAS login page.
   */
  async clickContactUsLink() {
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getContactUsLink(), true, timeout);
    await globalMethods.waitForClickable(await this.getContactUsLink(), timeout);
    //await globalMethods.click(await this.getContactUsLink(), 'Contact Us Link', timeout);  
    //Changed to Java script click because while executing in mobile, it is not able to click on Contact us link.
    await browser.executeScript('arguments[0].click();', await this.getContactUsLink());
    await browser.waitForAngularEnabled(true);
  }
  async enterFirstName(){}
  async enterLastName(){}
  async enterMiddleInitial(){}
  async enterDateOfBirth(){}
  async enterSex(){}
  async enterPrimaryPhoneNumber(){}
  async enterSecondaryPhoneNumber(){}
  async enterEmail(){}
  async enterAddress(){}
  async enterAddress2(){}
  async enterCity(){}
  async enterState(){}  
  async enterZip(){}  

  async enterContactInformation() {
    await this.enterFirstName();
    await this.enterLastName();
    await this.enterMiddleInitial();
    await this.enterDateOfBirth();
    await this.enterSex();
    await this.enterPrimaryPhoneNumber();
    await this.enterSecondaryPhoneNumber();
    await this.enterAddress();
    await this.enterAddress2();
    await this.enterCity();
    await this.enterState();
    await this.enterZip();
  }

  //Procedure
  /**
   * login method calls procedureEnterSignIn and clicks on login. This works for both desktop and mobile view port.
   * @param userName 
   * @param password 
   */
  async login(userName, password) {
    await this.procedureEnterSignIn(userName, password);
    await this.clickLogin();
  }
  /**
   * verifyDisplayOfSignInPage method verifies for presence of an element in sign in page.
   */
  async verifyDisplayOfSignInPage(){
    await browser.waitForAngularEnabled(false);
    await globalMethods.log("Verifying Sign In Page");
    await globalMethods.verifyElement(await this.getUserName(), timeout, true);
    await browser.waitForAngularEnabled(true);
  }
}
