import { browser, by, element } from 'protractor';
import '../global/global';
import * as globalMethods from 'quest-global';
import { AccountSettings } from './account-settings.po';
import { protractor } from 'protractor/built/ptor';
let timeout=20000;
var cartPagetext = '/cart';
var ordersPageText = '/orders';
var QuestDirectHomePageText = 'https://citqa.mq.questdiagnostics.com/';

let accountSettings: AccountSettings = new AccountSettings();

export class Navigation {
  // Getter Functions:

  getContactQuestButton() {
    return element(by.id('contact-quest'));
  }
  getSettingsAndBillings() {
    return element(by.id('settings-and-billings'));
  }
  getAccountSettings() {
    return element(by.id('account-settings'));
  }
  getLogout(){
    return element(by.id('logOut'));
  }
  getFAQ() {
    return element(by.id('faq'));
  }
  navigateToHome() {
    return '/';
  }
  signInButton() {
    return element(by.id('pl_SignIn_btn'));
  }
  questLogo() {
    return element(by.id('db_QuestLogo_img'));
  }
  authContactQuest() {
    return element(by.id('contact-quest'));
  }
  getAppointmentSchedulingMobileOptions() {
    return element(by.id('appointment-scheduling'));
  }
  getAppointmentScheduling() {
    return element(by.id('mq_ScheduleAppointmentMenu_button'));
  }
  payBills() {
    return element(by.id('pl_PayBills_btn'));
  }
  mobilePayBills() {
    return element(by.id('pay-bills'));
  }
  homeContactQuest() {
    return element(by.id('pl_ContactQuest_btn'));
  }
  FAQButton() {
    return element(by.id('pl_FAQ_btn'));
  }
  messageQuestButton() {
    return element(by.id('pl_MessageQuest_btn'));
  }
  authDropDownBillPay() {
    return element(by.id('pay-bills'));
  }
  mobileMessageQuest() {
    return element(by.id('contactDialog'));
  }
  mobileFAQ() {
    return element(by.id('faq'));
  }
  getUserAvatarMobile() {
    return element(by.id('options'));
  }
  messageQuestFirstName() {
    return element(by.id('mq_ContactFirstName_txt'));
  }
  messageQuestLastName() {
    return element(by.id('mq_ContactLastName_txt'));
  }
  messageQuestEmail() {
    return element(by.id('mq_ContactEmail_txt'));
  }
  messageQuestConfirmEmail() {
    return element(by.id('mq_ContactConfirmEmail_txt'));
  }
  messageQuestContactReason() {
    return element(by.id('mq_ContactReason_drp'));
  }
  messageQuestContactMessage() {
    return element(by.id('mq_ContactMessage_txt'));
  }
  messageQuestSendButton() {
    return element(by.id('mq_ContactSend_btn'));
  }
  getNotificationBell() {
    return element(by.id('notifications'));
  }
  
  navigationNotificationVerifyButton() {
    return element(by.id('not_Action_btn'));
  }
  getNotificationViewMoreButton() {
    return element(by.id('not_ViewMore_btn'));
  }
  getNotificationDelete() {
    return element(by.id('not_Dismiss_btn0'));
  }
  navigationNotificationActionsRequired() {
    return element(by.id('not_ActionRequired_txt'));
  }
  navigationNotificationHeader() {
    return element(by.id('not_Notifications_txt'));
  }
  getCart(){
    return globalMethods.getElementByXpath("//a[@id='cart']");
  }
  getPurchaseOrders(){
    return globalMethods.getElementById("purchase-orders");
  }
  getPurchaseOrdersTitle() {
    return element(by.id('qd_TakeoverLayout_title'));
  }
  getCartItems(){
    //var cartItems = element(by.xpath("//section[@class='cart__contents']")).all(by.tagName('li'));
    var cartItems =  globalMethods.getElementByXpath("//div[@class='cart__contents-advanced-access ng-star-inserted']/ul").all(by.tagName('li'));
    return cartItems;
  }
  getApplicationSwitcher(){
    return element(by.id('qd_ApplicationSwitch_drp')).element(by.tagName('button'));
  }
  getMyQuestOption(){
    return element(by.id('qd_MQLink_drp'));
  }
  selectResultCardMenu() {
    return element(by.id('rs_ResultCardMenu_btn0'));
  }
  getCheckout(){
    return element(by.id('cart_Checkout_btn'));
  }
  getAppointmentSchedulingArrowMobile(){
    return globalMethods.getElementByXpath("//a[@id='appointment-scheduling']/mat-icon");
  }
  //Setters
  /**
   * clickContactQuestButton method clicks on Contact Quest option of MyQuest Dashboard page.
   */
  async clickContactQuestButton(): Promise<void>{
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getContactQuestButton(), true, timeout);
    //await globalMethods.click(await this.getContactQuestButton(), 'Contact Quest Button', timeout);
    // Changed to java script click as normal click is not working consistently in mobile viewport.
    await browser.executeScript('arguments[0].click();', await this.getContactQuestButton());
    await browser.waitForAngularEnabled(true);
}
  async clickFAQ(): Promise<void>{
    await globalMethods.click(await this.getFAQ(), 'FAQ', timeout);
  }
  async clickMessageQuest(): Promise<void>{
    await globalMethods.click(await this.messageQuestButton(), 'Message Quest Button', timeout);
  }
  async clickAppointmentScheduling(): Promise<void>{
    var browserSize = await globalMethods.getConfigBrowserSize();
        await globalMethods.log("Browser Size Parameters:" + await browserSize);

        //If browser width/height is a desktop viewport:
        //click Pay Bills outright
        if (browserSize === 'md' || browserSize === 'lg') {
          await globalMethods.click(await this.getAppointmentScheduling(), 'Appointment Scheduling', timeout);
          return;
        }

        //If browser size is smaller than standard viewport:
        //two step process to click Pay Bills
        await globalMethods.click(await this.getUserAvatarMobile(), "User dropdown on mobile", timeout);
        await this.getAppointmentSchedulingMobileOptions();
        await globalMethods.click(await this.getAppointmentSchedulingMobileOptions(), 'Appointment Scheduling mobile', timeout);
  }
  /**
   * clickSettingsAndBillings method clicks on user profile avatar in Dashboard page.
   * This works both for desktop and mobile view port.
   */
  async clickSettingsAndBillings() {
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getSettingsAndBillings(), true, timeout);
    await globalMethods.mouseMove(await this.getSettingsAndBillings(), 'User Menu', timeout);
    await browser.waitForAngularEnabled(true);
  }
  /**
   * clickAccountSettings method clicks on Account Settings link available under user profile avatar.
   */
  async clickAccountSettings() {
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getAccountSettings(), true, timeout);
    await globalMethods.click(await this.getAccountSettings(), 'Account Settings', timeout);
    //Verification that we are on settings page
    await globalMethods.waitForElement(await accountSettings.getAccountSettingsTitle(), true, timeout);
    await browser.waitForAngularEnabled(true);
  }

  async clickLogout(): Promise<void>{
    await this.clickSettingsAndBillings();
    await browser.waitForAngularEnabled(false);
    await globalMethods.log("Clicked on User Menu");
    await globalMethods.waitForElement(await this.getLogout(), true, timeout);
    await globalMethods.log("Logout element is visible to click");
    //await globalMethods.click(await this.getLogout(), "Log Out link", timeout);
    await browser.executeScript('arguments[0].click();', await this.getLogout()); 
    await browser.waitForAngularEnabled(true);
}
/**
 * clickNotificationBell method clicks on notification bell icon on Dashboard page.
 * This method supports for Desktop and mobile view port.
 */
  async clickNotificationBell(): Promise<void>{    
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getNotificationBell(), true, timeout);
    await globalMethods.mouseMove(await this.getNotificationBell(), 'Notification Bell', timeout);
    await browser.waitForAngularEnabled(true);
}
/**
 * clickNotificationViewMore method clicks on notification view more link on Dashboard page.
 * This method supports for Desktop and mobile view port.
 */
  async clickNotificationViewMore(): Promise<void>{
    await browser.waitForAngularEnabled(false);
    await globalMethods.click(await this.getNotificationViewMoreButton(), 'Notification View More Button', timeout);
    await browser.waitForAngularEnabled(true);
}
/**
 * clickNavigationNotificationVerifyButton method clicks on Verify Button of notification.
 * This supports for desktop and mobile.
 */
  async clickNavigationNotificationVerifyButton(): Promise<void>{
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.navigationNotificationVerifyButton(), true, timeout);
    await globalMethods.waitForVisible(await this.navigationNotificationVerifyButton(), timeout);
    await globalMethods.click(await this.navigationNotificationVerifyButton(), 'Notification Verify Button in Sub account Page', timeout);
    await browser.waitForAngularEnabled(true);
  }
  async clickCart(): Promise<void>{
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getCart(), true, timeout);
    await globalMethods.click(await this.getCart(), 'Cart', timeout);
    await browser.waitForAngularEnabled(true);
  }
  /**
   * clickPurchaseOrders method clicks on purchase orders option in menu.
   */
  async clickPurchaseOrders() {
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getPurchaseOrders(), true, timeout);
    await globalMethods.waitForElement(await this.getCart(), true, timeout);
    await globalMethods.click(await this.getPurchaseOrders(), 'Purchase Orders', timeout);
    await browser.waitForAngularEnabled(true);
  }
  async clickApplicationSwitcher(): Promise<void> {
    await globalMethods.click(await this.getApplicationSwitcher(), 'Application Switcher', timeout);
  }
  async clickMyQuest(): Promise<void> {
    await globalMethods.click(await this.getMyQuestOption(), 'MyQuest Switcher', timeout);
  }
  //  Navigation Functions
  
  async navigateToHomePage() {
    var nav = false;
    var config = await browser.getProcessedConfig();
    var baseUrl = await config.baseUrl;
    try{
        let url = await browser.getCurrentUrl();
        console.log("Url:" + url);
        if(url !== baseUrl){
          //if(url !== 'https://mq2master.mq.questdiagnostics.com/web/home'){
         // if(url !== 'https://mq2lt.mq.questdiagnostics.com/web/home'){
            nav = true;
        }
    }
    catch(err){
        nav = true;
        console.log("Error getting url");
    }
    if(nav === true){
        browser.ignoreSynchronization = true;
        console.log("Ignore Synchronization set to true");
        await browser.get('/');
        browser.ignoreSynchronization = false;
        console.log("Ignore Synchronization set to false");
    }
  }
  async removeAllItemsFromCart(){
    await this.clickCart();
    await globalMethods.waitForElement(await this.getCheckout(), true, timeout);
    await globalMethods.verifyElement(await this.getCheckout(), timeout, true);
    await globalMethods.log("user is on cart page");
    //setting synchronization to true as user will be on the cart page
    // await browser.waitForAngularEnabled(true);

    let cartItems = this.getCartItems();
    var count = await cartItems.count();  
    await globalMethods.log("Items in cart:"+count);
    if(count > 0){
    var i=1;
    var j= count;
    while(i <= count){
        await globalMethods.waitForVisible(await element(by.xpath("//section[@class='cart__contents']//li["+j+"]//div[@class='cart-product-card__remove']//button")),  timeout)
        var targetloc = "//section[@class='cart__contents']//li["+j+"]//div[@class='cart-product-card__remove']//button";
            console.log(targetloc);
            // Pradeep 3/22 - Eventhough the page can is fully loaded, and completely within the viewport, Chromedriver will refuse to click the element if the element is wrapped in a div or a span sometimes.
            // Therefore using SendKeys(protractor.Key.RETURN) instead of click
            await element(by.xpath(targetloc)).sendKeys(protractor.Key.RETURN); 
            await browser.sleep(2000);
            i++;
            j--;
    }
}else{
    await globalMethods.log("No items present in the cart:"+count);
}
await accountSettings.clickClose();
await browser.waitForAngularEnabled(false);
}
async clickUserAvatar(){
  await globalMethods.click(await this.getUserAvatarMobile(), 'User Avatar', timeout);
}
async verifyAppointmentSchedulingArrowOnMobile(){
  await browser.waitForAngularEnabled(false);
  await globalMethods.waitForElement(await this.getAppointmentSchedulingArrowMobile(), true, timeout);
  await globalMethods.verifyElement(await this.getAppointmentSchedulingArrowMobile(), timeout, true);
  await browser.waitForAngularEnabled(true);
}

/**
 * verifyYourCartPage method verifies the url of Your Cart page.
 */
async verifyYourCartPage(){
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getCheckout(), true, timeout);
    var currentUrl = await browser.getCurrentUrl();
    await globalMethods.log("url" + currentUrl);
    await globalMethods.verifyTextContains(currentUrl, cartPagetext, true);
    await browser.waitForAngularEnabled(true);
}
/**
 * verifyPurchaseOrdersPage method verifies for an element in Purchase Orders page
 */
async verifyPurchaseOrdersPage(){
  await browser.waitForAngularEnabled(false);
  await globalMethods.waitForElement(await this.getPurchaseOrdersTitle(), true, timeout);
  await globalMethods.verifyElement(await this.getPurchaseOrdersTitle(), timeout,true);
  await browser.waitForAngularEnabled(true);
}
async verifyPurchaseOrdersPageURL(){
  await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getPurchaseOrdersTitle(), true, timeout);
    var currentUrl = await browser.getCurrentUrl();
    await globalMethods.log("url" + currentUrl);
    await globalMethods.verifyTextContains(currentUrl, ordersPageText, true);
    await globalMethods.verifyElement(await this.getPurchaseOrdersTitle(), timeout,true);
    await browser.waitForAngularEnabled(true);
}
/**
 * verifyQuestDirectHomePage method verifies the url of Quest Direct Home Page.
 */
async verifyQuestDirectHomePage(){
    await browser.waitForAngularEnabled(false);    
    await globalMethods.waitForElement(await this.getCart(), true, timeout);
    await browser.refresh();
    var currentUrl = await browser.getCurrentUrl();
    await globalMethods.log("url" + currentUrl);
    await globalMethods.verifyText(currentUrl, QuestDirectHomePageText, true);    
    await browser.waitForAngularEnabled(true);
}
}