import { by, element, browser, ExpectedConditions} from 'protractor';
import * as globalMethods from 'quest-global';
let timeout=9500;
export class ForgotUsername {

    //Getters

    getFirstName() {
        return element(by.id('firstName'));
    }

    getLastName() {
        return element(by.id('lastName'));
    }

    getEmailAddress() {
        return element(by.id('emailAddress'));
    }

    getDOB() {
        //return element(by.id('input_0'));
        return globalMethods.getElementById('dateOfBirth');
        //return element(by.name('dateOfBirth'));
    }

    getSubmitForgotUsername() {
        return element(by.id('btn-continue'));
    }

    getCancelForgotUsername() {
        //return element(by.xpath("//a[contains(text(),'Forgot Username')]"));
        return element(by.xpath("//a[text()='Cancel']"));
    }

    getForgotUsernameConfirmation() {
        return element(by.id('login-page'));
    }

    getForgotUsernameError() {
        return element(by.id('errorMsg'));
    }

    getForgotUsernameForm() {
        return element(by.id('fm1'));
    }

    getForgotUsernameSuccessMessage(){
        return element(by.xpath("//div[@class='login layout-align-start-center layout-column']"));
    }
    getBackTologin(){
        return element(by.xpath("//form[@id='fm']//a"));
    }

    //Reusable actions
    /*
     * enterForgotUsernameForm method gets the data from dataFile and enters data to the fields of forgot username page.
     * @param dataFile 
     */
    async enterForgotUsernameForm(dataFile){
        await globalMethods.waitForElement(await this.getForgotUsernameForm(), true, timeout);
        await globalMethods.log("Entering data in forgot username page");
        let firstname = await dataFile.firstName;
        let lastname = await dataFile.lastName;
        let emailaddress = await dataFile.email;
        var dob = await dataFile.dob;
               
        await globalMethods.waitForElement(await this.getFirstName(), true, timeout);
        await globalMethods.sendKeys(await this.getFirstName(), firstname, timeout, true);                
        await globalMethods.sendKeys(await this.getLastName(), lastname, timeout, true);
        await globalMethods.sendKeys(await this.getEmailAddress(), emailaddress, timeout, true);        
        await globalMethods.sendKeys(await this.getDOB(), dob, timeout, true);
    }
    /**
     * clickSubmit() method clicks on submit button of Forgot username page.
     */
    async clickSubmit(){
        await globalMethods.waitForElement(await this.getSubmitForgotUsername(), true, timeout);
        await globalMethods.waitForClickable(await this.getSubmitForgotUsername(), timeout);
        await globalMethods.log("Clicking Submit in Forgot Username page");
        //await browser.wait(ExpectedConditions.elementToBeClickable(await this.getSubmitForgotUsername()), timeout);
        await browser.executeScript('arguments[0].click();', await this.getSubmitForgotUsername());
    }
    /**
     * clickCancel() method clicks on cancel button of Forgot username page.
     */
    async clickCancel(){
        await browser.waitForAngularEnabled(false);
        await globalMethods.waitForElement(await this.getCancelForgotUsername(), true, timeout);        
        await globalMethods.log("Clicking cancel in forgot username page");
        await browser.executeScript('arguments[0].click();', await this.getCancelForgotUsername());
        await globalMethods.log("Clicked Cancel");
        await browser.waitForAngularEnabled(true);
    }
    /**
     * verifyForgotUsernamePage method verifies for an element to display in forgot username page.
     */
    async verifyForgotUsernamePage(){
        await browser.waitForAngularEnabled(false);
        await globalMethods.waitForElement(await this.getFirstName(), true, timeout);
        await globalMethods.verifyElement(await this.getFirstName(), timeout, true);
        await globalMethods.log("In Forgot Username page");
        await browser.waitForAngularEnabled(true);
    }
    /**
     * verifyResultMessageOnForgotUsernamePage method verifies for the message displayed on Forgot user name page for positive and negative flows.
     * It takes two parameters type of flow - positive or negative and result message displayed on the page.
     * @param positiveFlow 
     * @param resultMsg 
     */
    async verifyResultMessageOnForgotUsernamePage(positiveFlow, resultMsg){
        var expectedResult = resultMsg;
        if(positiveFlow === "Yes"){
        await globalMethods.waitForElement(await this.getBackTologin(), true, timeout);
        await globalMethods.verifyElement(await this.getBackTologin(), timeout, true);
        await globalMethods.log("BackTologin button present");
        var actualResult = await this.getForgotUsernameSuccessMessage().getText().then(async function(text){
            return await text.toString().trim().toLowerCase();
        });
        } 
        
        else {
        await globalMethods.waitForElement(await this.getForgotUsernameError(), true, timeout);
        await globalMethods.verifyElement(await this.getForgotUsernameError(), timeout, true);
            
        var actualResult = await this.getForgotUsernameError().getText().then(async function(text){
            return await text.toString().trim().toLowerCase();
        });
        }
        await globalMethods.log("Result Message from app: "+ actualResult);
        await globalMethods.log("Result Message from dataFile: " + expectedResult);
        await globalMethods.verifyTextContains(await actualResult, expectedResult, true);
        await browser.waitForAngularEnabled(true);
    }
}