import { browser, by, element } from 'protractor';
import * as globalMethods from 'quest-global';
import { CreateAccount } from '../createAccount.po';
import { Messages } from '../messages.po';

let timeout = 5000;
// const signInUrl = "https://lbqacas.qa.medplus.com/cas/login?service=https%3A%2F%2Fmq2master.mq.questdiagnostics.com%2Fmq-service%2Flogin%2Fcas&mqback=https%3A%2F%2Fmq2master.mq.questdiagnostics.com";
//const signInUrl = "https://lbqacas.qa.medplus.com/cas/login?service=https%3A%2F%2Fmq2lt.mq.questdiagnostics.com%2Fmq-service%2Flogin%2Fcas&mqback=https%3A%2F%2Fmq2lt.mq.questdiagnostics.com";
let createAccount: CreateAccount = new CreateAccount();
let messages: Messages = new Messages();
export class Home {

    // Getters

    getPayBills() {
        return element(by.id('pl_PayBills_btn'));
    }

    getPayBillsMobile() {
        return element(by.id('pay-bills'));
    }

    getOptionsMobile() {
        return element(by.id('options'));
    }

    getSignIn() {
        return element(by.id('pl_SignIn_btn'));
    }
    getSignInMobile() {
        // return element(by.id('sign-in'));
        return globalMethods.getElementById('relogin');
    }

    getAppointmentScheduling() {
        return element(by.id('mq_ScheduleAppointmentMenu_button'));
    }

    getHeader() {
        return element(by.id('pl_HeaderNewSite_txt'));
    }
    getContactQuest() {
        return element(by.id('pl_ContactQuest_btn'));
    }
    getFAQButton() {
        return element(by.id('pl_FAQ_btn'));
    }
    getLocationDetector() {
        return globalMethods.getElementById('psc-search-current-location-btn');
    }
    getCookiesNotice() {
        return globalMethods.getElementByXpath("//a[contains(text(),'Cookies Notice')]");
    }
    getPrivacy(){
        return globalMethods.getElementByXpath("//a[text() ='Privacy']");
    }
    getTerms(){
        return globalMethods.getElementByXpath("//a[contains(text(),'Terms')]");
    }
    getNonDiscriminationNotice(){
        return globalMethods.getElementByXpath("//a[contains(text(),'Non-Discrimination Notice')]")
    }
    getSocialMediaIcons(){
        return globalMethods.getElementByXpath("//a[@aria-label='social media']"); 
    }
    getQuestDiagnosticsFooterLogo(){
        return element(by.xpath("//div[@class='qd-footer__logo']//img[@id='db_QuestLogo_img']")); 
    }
    getHeroSpaceContent(){
        return globalMethods.getElementByXpath("//div[@class='pl-marketing-banner']");
     }
    // Reusable Methods

    /**
     * clickSignIn method clicks on SignIn button available on myquest home page. 
     * This method supports for desktop and mobile viewport.
     */
    async clickSignIn() {
        var browserSize = await globalMethods.getConfigBrowserSize();
        var config = await browser.getProcessedConfig();
        var signInUrl = await config.signInUrl;
    
        await globalMethods.log("Browser Size Params:" + browser.params.viewport);

        if (browserSize === 'md' || browserSize === 'lg') {
            await globalMethods.waitForElement(await this.getSignIn(), true, timeout);
            await globalMethods.click(await this.getSignIn(), 'Sign In', timeout)
        // Getting login page again here because when running multiple tests, login page sometimes does not load. Performing below will ensure login page always loads, 
        await browser.waitForAngularEnabled(false);
        console.log("Ignore Synchronization set to true");
        await globalMethods.log("sign In url from config file== " + signInUrl);
        await browser.get(signInUrl);
        await browser.waitForAngularEnabled(true);
        console.log("Ignore Synchronization set to false");
            return;

        }
        this.clickOptionsMobile();
        await globalMethods.waitForElement(await this.getSignInMobile(), true, timeout);
        await globalMethods.click(await this.getSignInMobile(), 'Mobile Sign In', timeout);
    }

    async clickOptionsMobile() {
        await globalMethods.click(await this.getOptionsMobile(), 'Mobile Options', timeout);
    }

    //Click Pay Bills on MyQuest Home Page
    async clickPayBills() {
        var browserSize = await globalMethods.getConfigBrowserSize();
        await globalMethods.log("Browser Size Parameters:" + browser.params.viewport);

        //If browser width/height is a desktop viewport:
        //click Pay Bills outright
        if (browserSize === 'md' || browserSize === 'lg') {
            await globalMethods.click(await this.getPayBills(), 'Pay Bills', timeout);
            return;
        }

        //If browser size is smaller than standard viewport:
        //two step process to click Pay Bills
        await this.clickOptionsMobile();
        await globalMethods.click(await this.getPayBillsMobile(), 'Pay Bills mobile', timeout);
    }

    async clickAppointmentScheduling() {
        await globalMethods.click(await this.getAppointmentScheduling(), 'Schedule Appointment', timeout);
    }
    /**
     * clickLocationDetector method clicks on Location Detector on MyQuest Home Page.
     */
    async clickLocationDetector(){
        await browser.waitForAngularEnabled(false);
        await globalMethods.waitForElement(await this.getLocationDetector(), true, timeout);
        await browser.executeScript('arguments[0].click();', await this.getLocationDetector());
        await browser.waitForAngularEnabled(true);
    }
    async clickCookiesNotice() {
        await globalMethods.waitForElement(await this.getCookiesNotice(), true, timeout);
        await globalMethods.click(await this.getCookiesNotice(), 'Cookies Notice', timeout);
    }
    async clickPrivacyLink() {
        await globalMethods.waitForElement(await this.getPrivacy(), true, timeout);
        await globalMethods.click(await this.getPrivacy(), 'Privacy', timeout);
    }
    async clickTermsLink() {
        await globalMethods.waitForElement(await this.getTerms(), true, timeout);
        await globalMethods.click(await this.getTerms(), 'Terms', timeout);
    }
    async clickNondiscriminationNoticeLink() {
        await globalMethods.waitForElement(await this.getNonDiscriminationNotice(), true, timeout);
        await globalMethods.click(await this.getNonDiscriminationNotice(), 'Non Descrimination Notice', timeout);
    }
    async clickSocialMediaLinks() {
        await globalMethods.waitForElement(await this.getSocialMediaIcons(), true, timeout);
        await globalMethods.click(await this.getSocialMediaIcons(), 'Social Media Icons', timeout);
    }
    async clickQuestDiagnosticsFooterLogo() {
        await globalMethods.waitForElement(await this.getQuestDiagnosticsFooterLogo(), true, timeout);
        await globalMethods.click(await this.getQuestDiagnosticsFooterLogo(), 'Quest Diagnostics Footer Logo', timeout);
    }
    /**
     * procedureClickFooterLinks method clicks on footer links available on MyQuest Home Page.
     * @param dataFile 
     */
    async procedureClickFooterLinks(dataFile){
    
    let CookiesNoticeLink= await dataFile.cookiesNoticeLink;
    let PrivacyLink= await dataFile.privacyLink;
    let TermsLink= await dataFile.termsLink;
    let NonDiscriminationNotice = await dataFile.nondiscrimination;
    let SocialMediaLogo= await dataFile.socialMediaLogo;
    let QuestDiagnosticsLogo= await dataFile.questDiagnosticsLogo;

    await browser.waitForAngularEnabled(false);
    if(CookiesNoticeLink){        
        await this.clickCookiesNotice();
    }
    if(PrivacyLink){       
        await this.clickPrivacyLink();
    }
    if(TermsLink){        
        await this.clickTermsLink();
    }
    if(NonDiscriminationNotice){
        await this.clickNondiscriminationNoticeLink();
    }
    if(SocialMediaLogo){
        await this.clickSocialMediaLinks();
    }
    if(QuestDiagnosticsLogo){
        await this.clickQuestDiagnosticsFooterLogo();
    }
    await browser.waitForAngularEnabled(true);
}
/**
 * procedureVerifyFooterLinks method verifies for the footer links available on MyQuest Home Page. This method gets data of cookiesNoticeText,  privacyText, termsText, nonDiscriminationText, socialMediaLogoText and questDiagnosticsLogoText from dataFile.
 * @param dataFile 
 */
async procedureVerifyFooterLinks(dataFile){
    let CookiesNoticeText = await dataFile.cookiesNoticeText;
    let PrivacyText = await dataFile.privacyText;
    let TermsText = await dataFile.termsText;
    let NonDiscriminationText = await dataFile.nonDiscriminationText;
    let SocialMediaLogoText= await dataFile.socialMediaLogoText;
    let QuestDiagnosticsLogoText = await dataFile.questDiagnosticsLogoText;

    let cookiesNotice = await dataFile.cookiesNoticePage;
    let Privacy = await dataFile.privacyPage;
    let Terms = await dataFile.termsPage;
    let NonDiscrimination = await dataFile.nondiscriminationPage;
    let SocialMediaLogo= await dataFile.socialMediaLogo;
    let QuestDiagnosticsLogo= await dataFile.questDiagnosticsLogo;

    await globalMethods.switchTabs(1).then(async function() {
    await browser.waitForAngularEnabled(false);
    if(cookiesNotice){
        await globalMethods.waitForElement(await messages.getClose(), true, timeout);
        var currentUrl = await browser.getCurrentUrl();
        await globalMethods.log("url:" + currentUrl);
        await globalMethods.verifyTextContains(currentUrl, CookiesNoticeText, true);
    }
    if(Privacy){
        await globalMethods.waitForElement(await messages.getClose(), true, timeout);
        var currentUrl = await browser.getCurrentUrl();
        await globalMethods.log("url:" + currentUrl);
        await globalMethods.verifyTextContains(currentUrl, PrivacyText, true);
    }
    if(Terms){
        await globalMethods.waitForElement(await messages.getClose(), true, timeout);
        var currentUrl = await browser.getCurrentUrl();
        await globalMethods.log("url:" + currentUrl);
        await globalMethods.verifyTextContains(currentUrl, TermsText, true);
    }
    if(NonDiscrimination){
        await globalMethods.waitForElement(await messages.getClose(), true, timeout);
        var currentUrl = await browser.getCurrentUrl();
        await globalMethods.log("url:" + currentUrl);
        await globalMethods.verifyTextContains(currentUrl, NonDiscriminationText, true);
    }
    if(SocialMediaLogo){
        await globalMethods.waitForElement(await messages.getClose(), true, timeout);
        var currentUrl = await browser.getCurrentUrl();
        await globalMethods.log("url:" + currentUrl);
        await globalMethods.verifyTextContains(currentUrl, SocialMediaLogoText, true);
    }
    if(QuestDiagnosticsLogo){
        await globalMethods.waitForElement(await messages.getClose(), true, timeout);
        var currentUrl = await browser.getCurrentUrl();
        await globalMethods.log("url:" + currentUrl);
        await globalMethods.verifyTextContains(currentUrl, QuestDiagnosticsLogoText, true);
    }
    
    });
       //await globalMethods.closeTab();
    var handles = await browser.driver.getAllWindowHandles()
     for (let i = 1; i < handles.length; i++) {
      if (handles[i]) {
       await globalMethods.log("Closing tab: " + i + " **");
       browser.driver.switchTo().window(handles[i]);
       await globalMethods.closeTab();
      }
     }     
    await globalMethods.switchTabs(0);
    await browser.waitForAngularEnabled(true);
   
}

/**
 * verifyUpdatedHerospaceContentOnMyQuestHomePage method verifies for the diaply of hero space content on MyQuest Home page.
 */
async verifyUpdatedHerospaceContentOnMyQuestHomePage(){
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getHeroSpaceContent(), true, timeout);
    await globalMethods.verifyElement(await this.getHeroSpaceContent(), timeout, true);
    await browser.waitForAngularEnabled(true);
}
}
