import { by, element, browser } from 'protractor';
import * as globalMethods from 'quest-global'
let timeout=9500;

export class ResetPassword {
  getCurrentPassword() {
    return element(by.id('oldPassword'));
  }
  getNewPassword() {
    return element(by.id('password'));
  }
  getConfirmPassword() {
    return element(by.id('confirmPassword'));
  }
//   Using the locator //form[@id='fm1']//button//span because when we use only till //button, area under selection is more than the button area in UI. Therefore using till //span which selects text present inside the button
  getChangePasswordButton() {
    return element(by.xpath("//form[@id='fm1']//button//span"));
  }
  getCancelButton() {
    return element(by.id('beforehelp'));
  }
  getPasswordInfoErrorMsg() {
    return element(by.id('msg'));
  }
  /**
   * clickChangePasswordButton method clicks on 'Change Password' of Account Settings page.
   */
  async clickChangePasswordButton(): Promise<void> {
    await browser.waitForAngularEnabled(false);
    await globalMethods.click(await this.getChangePasswordButton(), 'Change Password Button', timeout);
    await browser.waitForAngularEnabled(true);
    
  }
  /**
   * clickCancel method clicks on cancel button of 'Reset Password' page.
   */
  async clickCancelButton(): Promise<void> {
    await browser.waitForAngularEnabled(false);
    await globalMethods.click(await this.getCancelButton(), 'Cancel Button', timeout);
    await browser.waitForAngularEnabled(true);
  }
  /**
   * enterDataOnResetPasswordPage method enters currentPassword, NewPassword,, ConfirmPassword in respective fields of Reset Password page.
   * @param currentPassword 
   * @param newPassword 
   * @param confirmPassword 
   */
  async enterDataOnResetPasswordPage(currentPassword, newPassword, confirmPassword){
  if(currentPassword != ""){
      await globalMethods.sendText(await this.getCurrentPassword(), currentPassword, timeout);
  }
  if(newPassword != ""){
      await globalMethods.sendText(await this.getNewPassword(), newPassword, timeout);
  }
  if(confirmPassword != ""){
      await globalMethods.sendText(await this.getConfirmPassword(), confirmPassword, timeout);
  }
  }
  /**
   * verifyPasswordErrorMessage method verifies for the invalid error message of Reset Password page.
   * @param errorMsg 
   */
  async verifyPasswordErrorMessage(errorMsg){
    await browser.waitForAngularEnabled(false);
    await globalMethods.waitForElement(await this.getPasswordInfoErrorMsg(), true, timeout);
    // Sowjanya Aug 9: Adding sleep here bcos, in edge browser the execution is so fast and could get the message from application while results in failure.
    await browser.sleep(5000);
    var actualPasswordInfoErrorMsg = await this.getPasswordInfoErrorMsg().getText();
    //await globalMethods.log("Error Message =>"+actualPasswordInfoErrorMsg);    
    await globalMethods.verifyFailOnError(await actualPasswordInfoErrorMsg.trim(), errorMsg, 'Error Message matches', true);
    await browser.waitForAngularEnabled(true);
  }
}