import { Then, When, Given } from 'cucumber';
import { appendFile } from 'fs';
import { browser } from 'protractor';
import * as globalMethods from 'quest-global';
import { Common } from '../support/common';
import { ApiAutomated } from '../api/apiAutomated';
import { DbConnect } from '../api/dbConnect';
import { data } from 'jquery';
import * as hcspayloadstandardval from '../../jsonfiles/hcspayloadstandardval.json';
import * as hcsuserdata from '../../jsonfiles/hcsuserdata.json';
import * as hcslengthypayload from '../../jsonfiles/hcslengthypayload.json';
import * as hcspayloadjson from '../../jsonfiles/hcspayload.json';
import * as hrspayloadjson from '../../jsonfiles/hrspayload.json';

let axios = require('axios');
let global = require('../global/global');
let apiAutomated = new ApiAutomated();
let dbConnect = new DbConnect();
let timeout=9500;
var apiUrl: String;
let response;
let status;
let statuskey;
let errormessage;
let rnum = Math.floor(Math.random()*(999-100+1)+100);

Given('HCS API request', async()=> {
    apiUrl = "https://health-customer.gss.qa.aws.qdx.com";
})
//create customer POST
When('I do POST for create customer for HCS', verifyHCSPOST);
async function verifyHCSPOST() {
    let guid = await apiAutomated.newGuid();     
    (hcspayloadjson.externalId as string) = guid;
    (hcspayloadjson.ucdmid as string) = guid;
    (hcspayloadjson.tpsPersonId as unknown)= rnum;    
    response = await apiAutomated.checkPOSTStatus(apiUrl + "/customer", hcspayloadjson);
}
Then('I get 201 created response for HCS', verifyHCSPOSTResponse);
 async function verifyHCSPOSTResponse() {
    await apiAutomated.verifyresponse(response.status, 201);    
}
//repost the same message twice, should get 409 conflict
When('I do same POST for create customer', verifyHCSRepost);
async function verifyHCSRepost() {
    errormessage = await apiAutomated.checkErrorPOSTAPI(apiUrl + "/customer", hcspayloadstandardval);
}
Then('I get 409 conflict response', verifyHCSRepostResponse);
 async function verifyHCSRepostResponse() {
     await apiAutomated.verifyresponse(errormessage, 409);   
}
//Add userdata to create new customer
When('I add user data for new customer', verifyAddUserData);
async function verifyAddUserData() {
   response = await apiAutomated.checkPOSTStatus(apiUrl + "/userdata/cdd09ef8-7cc4-409a-9d27-1d31619bb59f/fifth"+ rnum, hcsuserdata);
}
Then('I get 201 created response for user data', verifyAddUserDataResponse);
 async function verifyAddUserDataResponse() {
     await apiAutomated.verifyresponse(response.status, 201);   
}
//Add same userdata twice, should get conflict error
When('I add same user data with same key', verifyAddUserDataRepost);
async function verifyAddUserDataRepost() {
    errormessage = await apiAutomated.checkErrorPOSTAPI(apiUrl + "/userdata/cdd09ef8-7cc4-409a-9d27-1d31619bb59f/fifth", hcsuserdata);       
}
Then('I get 409 conflict response for same key', verifyAddUserDataRepostResponse);
 async function verifyAddUserDataRepostResponse() {
     await apiAutomated.verifyresponse(errormessage, 409);   
}
//Verify userdata with lengthy string
When('I add user data with lengthy string', verifyPOSTLengthyKey);
async function verifyPOSTLengthyKey() {
    status = await apiAutomated.checkErrorPOSTAPI(apiUrl + "/userdata/cdd09ef8-7cc4-409a-9d27-1d31619bb59f/Eight", hcslengthypayload);
}
Then('I should get 400 for lengthy string', verifyPOSTLengthyKeyResponse);
 async function verifyPOSTLengthyKeyResponse() {
     await apiAutomated.verifyresponse(status, 400);   
}
//GET userdata wth userid
When('I GET user data with userid', verifyGETUserData);
async function verifyGETUserData() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/userdata/cdd09ef8-7cc4-409a-9d27-1d31619bb59f");
}
Then('I should get 200 response with same userid', verifyGETUserDataResponse);
 async function verifyGETUserDataResponse() {
     await apiAutomated.verifyresponse(response.status, 200);   
}
//GET userdata with specific key
When('I GET user data with key', verifyGETUserDataSpecificKey);
async function verifyGETUserDataSpecificKey() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/userdata/cdd09ef8-7cc4-409a-9d27-1d31619bb59f/fifth");
}
Then('I should get response with user data with specific key', verifyGETUserDataSpecificKeyResponse);
 async function verifyGETUserDataSpecificKeyResponse() {
     await apiAutomated.verifyresponse(response.status, 200);   
}
//PUT userdata to add or update 
When('I PUT to Replace newly added value for specific key', verifyPUTUserData);
async function verifyPUTUserData() {
    response = await apiAutomated.checkPUTStatus(apiUrl + "/userdata/cdd09ef8-7cc4-409a-9d27-1d31619bb59f/fifth", hcsuserdata);
}
Then('I should get response with new data', verifyPUTUserDataResponse);
 async function verifyPUTUserDataResponse() {
     await apiAutomated.verifyresponse(response.status, 200);   
}
