import { Then, When, Given } from 'cucumber';
import { appendFile } from 'fs';
import { browser } from 'protractor';
import * as globalMethods from 'quest-global';
import { Common } from '../support/common';
import { ApiAutomated } from '../api/apiAutomated';
import { DbConnect } from '../api/dbConnect';
import { data } from 'jquery';
import * as labreportjson from '../../jsonfiles/labreport.json';
import * as hrspayloadjson from '../../jsonfiles/hrspayload.json';

let axios = require('axios');
var chai = require('chai');
var assert = chai.assert;
var crypto = require('crypto');
let global = require('../global/global');
let apiAutomated = new ApiAutomated();
let dbConnect = new DbConnect();
let timeout=9500;
let apiUrl: String;
let response;
let status;

//Same given resused for all steps
Given('HRS API request', async()=> {
    apiUrl = "https://health-results.gss.qa.aws.qdx.com";
})
//verify Variant response
When('I do GET request to /reports', verifyVariantSpecProperties);
async function verifyVariantSpecProperties() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/reports?testGuid=ef541747-39cc-4713-aeb1-039d65280b69");
}
// verify must have properties of variant - geneName, hgvsTranscript, hgvsProtein 
Then('I get Variant response', verifyVariantSpecPropertiesResponse);
async function verifyVariantSpecPropertiesResponse() {
    var results = response.data[0].results;
        for (var i = 0; i < results.length; i++) {
            var resultStatus = results[i].resultStatus;
            if (resultStatus == "EDGECASE" || resultStatus == "POSITIVE") {
                var variants = results[i].variants;
                for (var j = 0; j < variants.length; j++) {
                    assert(variants[j] != null, "variants should not be null");
                    assert(variants[j].geneName != null, "gene name should not be null");
                    assert(variants[j].hgvsTranscript != null, "hgvsTranscript should not be null");
                    assert(variants[j].hgvsProtein != null, "hgvsProtein should not be null");
                }
            }
        }  
}
//Connect to DB and get all rows
Then('connect to DB to get all rows', function(callback){
    dbConnect.connectToDatabase(callback, 'select * from HealthResults.variant', './jsonfiles/dbdata.xlsx');
    callback;
});

//Verify GET HRS for testGuid with pwn_status as APPROVED
When('I do GET request to /reports for pwn results status APPROVED', verifyPWNStatusApproved);
async function verifyPWNStatusApproved() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/reports?testGuid=98428105-1574-4b81-b38e-49fae09cfb90");
}
Then('I get HRS response with status as APPROVED', verifyPWNStatusApprovedResponse);
async function verifyPWNStatusApprovedResponse() {
    var status = response.data[0].status;        
                    assert(status= "APPROVED", "pwn_status APPROVED is not displayed in response status");                   
}

//Verify GET HRS for testGuid with pwn_status as SESSION_PENDING
When('I do GET request to /reports for pwn results status SESSION PENDING', verifyPWNStatusSessionPending);
async function verifyPWNStatusSessionPending() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/reports?testGuid=8c170209-c8d9-4d81-972a-958d1c4913f0");
}
Then('I get HRS response with status as ON_HOLD', verifyPWNStatusSessionPendingResponse);
async function verifyPWNStatusSessionPendingResponse() {
    var status = response.data[0].status;        
                    assert(status= "ON_HOLD", "pwn_status ON_HOLD is not displayed in response status");                   
}

//For HRS Post 
 When('I do POST v1 reports', verifyPOSTHRSV1Report);
 async function verifyPOSTHRSV1Report() {  
    // status = await apiAutomated.verifyHRSPOSTtV1Reports(apiUrl +"/v1/reports");
    let resVal = await apiAutomated.makeRandomString(63);
        console.log("*****resVal*****", resVal);
        labreportjson.order.tests[38477].resultData[86025898].data.resultValue = resVal;
        var base64LabReport = await Buffer.from(JSON.stringify(labreportjson)).toString('base64');        
        var md5LabReport = crypto.createHash('md5').update(JSON.stringify(labreportjson)).digest('hex');
        console.log ("*****md5***** ", md5LabReport);
        (hrspayloadjson.json as string) = base64LabReport;
        (hrspayloadjson.jsonMd5 as string) = md5LabReport;
    response = await apiAutomated.checkPOSTStatus(apiUrl +"/v1/reports",hrspayloadjson);
    console.log("**********reponsestatus***********"+ response.status);
} 
 Then('labreport gets created with correct response', verifyPOSTHRSV1ReportResponse);
 async function verifyPOSTHRSV1ReportResponse() {
     await apiAutomated.verifyresponse(response.status, 201);  
}
// DELETE the new record for HRS POST    
 When('I do DELETE should delete the existing record with HRS POST', verifyHRSDelete);
 async function verifyHRSDelete() {  
    response = await apiAutomated.checkDeleteAPI(apiUrl+"/testing/delete?adnaProcessId=RPS7G8L4R4R3M8T");
}
 Then('I get success response', verifyHRSDeleteResponse);
 async function verifyHRSDeleteResponse() {
     await apiAutomated.verifyresponse(response, true);  
}
