import { Then, When, Given, JsonFormatter } from 'cucumber';
import { appendFile } from 'fs';
import { browser } from 'protractor';
import * as globalMethods from 'quest-global';
import { Common } from '../support/common';
import { ApiAutomated } from '../api/apiAutomated'; 
import * as hisjson from '../../jsonfiles/his.json';

let axios = require('axios');
let global = require('../global/global');
let apiAutomated = new ApiAutomated();
let timeout=9500;
let apiUrl: String;
let devapiUrl: String;
let response;
let status;

Given('IQDB API request in qa environment', async()=> {
    apiUrl = "https://api.iqdb.sqa.qdx.com";
})
//GET request for genetic review with pwnid
When('I do GET request to /v1/eins/wip/summary', verifyGETnumberOfBatches);
async function verifyGETnumberOfBatches() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/eins/wip/summary");
}
Then('I get response with number of TechReview Pending', verifyGETNoOfBatches);
async function verifyGETNoOfBatches() {
    let expected = "{\"totalBatch\":\"11\"}";
    await apiAutomated.verifyresponse(JSON.stringify(response.data), expected);
}
//GET request for genetic review with ParentId
When('I do GET request to /geneticreviewpid', verifyGETGeneticReviewPId);
async function verifyGETGeneticReviewPId() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v2/orders/geneticreview?ptId=bd648003-310d-4487-8700-dfca5d2a327d");
}
Then('I get response with approvalStatus approved pid', verifyGETGeneticReviewPIdResponse);
async function verifyGETGeneticReviewPIdResponse() {
    let expected = "{\"orderId\":6073827,\"approvalStatus\":\"APPROVED\",\"approvalTimestamp\":\"2022-01-18 19:19:48\",\"messageVariant\":\"VARIANT5\"}";
    await apiAutomated.verifyresponse(JSON.stringify(response.data), expected);
}
//GET request for health-history with ParentId
When('I do GET for health-history request for parent-test-guid', verifyHISHealthHistoryCheck);
async function verifyHISHealthHistoryCheck() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/health-history/bd648003-310d-4487-8700-dfca5d2a327d");
}
Then('I get response data for parent-test-guid', verifyHISHealthHistoryCheckResponse);
async function verifyHISHealthHistoryCheckResponse() {
    await apiAutomated.verifyresponse(response.status, 200); 
}
//GET request for health-history with ParentId with disease category filter
When('I do GET for health-history request for disease category', verifyHISHealthHistoryCheckqc);
async function verifyHISHealthHistoryCheckqc() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/health-history/bd648003-310d-4487-8700-dfca5d2a327d?dc=carrier");
}
Then('I get response data for disease category', verifyHISHealthHistoryCheckqcResponse);
async function verifyHISHealthHistoryCheckqcResponse() {
    await apiAutomated.verifyresponse(response.status, 200); 
}
//GET request for health-history with ParentId with question key filter
When('I do GET for health-history request filter by question key', verifyHISHealthHistoryCheckqc);
async function verifyHISHealthHistoryCheckqk() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/health-history/bd648003-310d-4487-8700-dfca5d2a327d?qk=geneticConditionPersonalHistory");
}
Then('I get response data filter by question key', verifyHISHealthHistoryCheckqcResponse);
async function verifyHISHealthHistoryCheckqkResponse() {
    await apiAutomated.verifyresponse(response.status, 200); 
}
//GET request for health-history with ParentId with multiple question keys filter
When('I do GET for health-history request for multiple question keys', verifyHISHealthHistoryCheckmultipleQK);
async function verifyHISHealthHistoryCheckmultipleQK() {
    response = await apiAutomated.checkGETStatus(apiUrl + "/v1/health-history/19b08e0e-e715-444e-90c2-29782a2a8674?qk=hereditaryCancerSyndromePersonalHistory&qk=cardioPersonalHistory");
}
Then('I get response data for multiple question keys', verifyHISHealthHistoryCheckmultipleQKResponse);
async function verifyHISHealthHistoryCheckmultipleQKResponse() {
    await apiAutomated.verifyresponse(response.status, 200); 
}
//GET to check status of API 
When('I do GET request to /status', verifyHISStatus);
async function verifyHISStatus() {
    response = await apiAutomated.checkGETStatus(apiUrl +"/health-integration-feature/status");
}
Then('I get response code 200', verifyHISResponse);
async function verifyHISResponse() {
    let expected = "{\"health-integration-queues\":\"disabled\",\"health-integration-pwn-communication\":\"enabled\"}";
    await apiAutomated.verifyresponse(response.data, expected);
}
//GET to check healthstatus of API 
When('I do GET for healthcheck request', verifyHISHealthCheck);
async function verifyHISHealthCheck() {
    response = await apiAutomated.checkGETStatus(apiUrl +"/healthcheck");
}
Then('I get response data', verifyHISHealthCheckResponse);
async function verifyHISHealthCheckResponse() {
    let expected = "\"Service status is healthy.\"";
    await apiAutomated.verifyresponse(response.data, expected);  
}
